#include "TDC7200.h"

#define PIN_TDC7200_INT       1 // D1 - PA10
#define PIN_TDC7200_ENABLE    3 // D3 - PA09
//#define PIN_TDC7200_STOP      5
//#define PIN_TDC7200_START     6

#define PIN_TDC7200_SPI_CS    10 // D10 - PA18
#define TDC7200_CLOCK_FREQ_HZ 8000000

static TDC7200 tof(PIN_TDC7200_ENABLE, PIN_TDC7200_SPI_CS, TDC7200_CLOCK_FREQ_HZ);


#define MAX_NUM_STOPS 5
uint8_t tdc7200_stops = 5;
uint8_t tdc7200_mode = 2;
uint8_t tdc7200_avg = 1;
uint8_t tdc7200_cal = 10;

// Program setup

void setup()
{
  SerialUSB.begin(115200);

//  while (!SerialUSB) {
//    ; // wait for serial port to connect
//  }

//    SerialUSB.println(F("-- Starting TDC7200 test --"));
    //SerialUSB.println(F_CPU);

  //  pinMode(PIN_TDC7200_ENABLE, OUTPUT);
  //  digitalWrite(PIN_TDC7200_ENABLE, HIGH);
  //
  //zz: goto zz;


  while (not tof.begin())
  {
    SerialUSB.println(F("Failed to init TDC7200"));
    delay(1000);
  }

  pinMode(PIN_TDC7200_INT, INPUT_PULLUP);     // active low (open drain)

//  digitalWrite(PIN_TDC7200_START, LOW);
//  pinMode(PIN_TDC7200_START, OUTPUT);

//  digitalWrite(PIN_TDC7200_STOP, LOW);
//  pinMode(PIN_TDC7200_STOP, OUTPUT);

  if (not tof.setupMeasurement( tdc7200_cal,         // cal2Periods
                                tdc7200_avg,          // avgCycles
                                tdc7200_stops,  // numStops
                                tdc7200_mode ))        // mode
  {
    SerialUSB.println(F("Failed to setup measurement"));
    while (1);
  }

  // Setup stop mask to suppress stops during the initial timing period.
  //    tof.setupStopMask(121000000ull);

  // Setup overflow to timeout if tof takes longer than timeout.
  //    tof.setupOverflow(Stop0000000ull);
}

// This is to avoid a bug when 0 is not displayed with standard print or println
inline void print_int64_t(int64_t n)
{
  if (n == 0)
    SerialUSB.print("0");
  else
    SerialUSB.print(n);
}

// EEPROM

void print_saved_vars()
{
  SerialUSB.print("TDC7200 stops: "); SerialUSB.println(tdc7200_stops);
  SerialUSB.print("TDC7200 mode: "); SerialUSB.println(tdc7200_mode);

  SerialUSB.println("? for help");
}

// Command interpreter

void cmd_interpret(const char* cmd, int cmd_len)
{
  // HELP
  if (
    ((cmd_len == 1) && (!strcmp(cmd, "?"))) ||
    ((cmd_len == 4) && (!strcmp(cmd, "help")))
  )
  {
    SerialUSB.println("-----------------------------");

    SerialUSB.println("stops<N> - set number of TDC7200 stop signals to N");
    SerialUSB.println("mode<N> - set mode of TDC7200 (1 or 2)");

    print_saved_vars();
    SerialUSB.println("-----------------------------");

  }
  else

  // STOPS
  if ( (cmd_len >= 6) && (!strncmp(cmd, "stops", 5)))
  {
    int tmp_stops = atoi(cmd + 5);

    if ( (tmp_stops>=1) && (tmp_stops<=MAX_NUM_STOPS))
    {
    
      tdc7200_stops = tmp_stops;
      
      SerialUSB.print("Set stops to ");
      SerialUSB.println(tdc7200_stops);


      if (not tof.setupMeasurement( tdc7200_cal,         // cal2Periods
                                    tdc7200_avg,          // avgCycles
                                    tdc7200_stops,  // numStops
                                    tdc7200_mode ))        // mode
      {
        SerialUSB.println(F("Failed to setup measurement"));
        while (1);
      }
  
    }
    else
    {
      SerialUSB.print("Parameter should be 1..");
      SerialUSB.println(MAX_NUM_STOPS);
    }
   
  }
  else

  // MODE
  if ( (cmd_len >= 5) && (!strncmp(cmd, "mode", 4)))
  {
    int tmp_mode = atoi(cmd + 4);

    if ( (tmp_mode>=1) && (tmp_mode<=2))
    {
    
      tdc7200_mode = tmp_mode;
      
      SerialUSB.print("Set mode to ");
      SerialUSB.println(tdc7200_mode);


      if (not tof.setupMeasurement( tdc7200_cal,         // cal2Periods
                                    tdc7200_avg,          // avgCycles
                                    tdc7200_stops,  // numStops
                                    tdc7200_mode ))        // mode
      {
        SerialUSB.println(F("Failed to setup measurement"));
        while (1);
      }
  
    }
    else
    {
      SerialUSB.println("Parameter should be 1 or 2");
    }
  }
  
}

// Program main loop

void loop()
{
  static boolean meas_started = false;

  // Start the measurement

  if (meas_started == false)
  {
    tof.startMeasurement();
    meas_started = true;
    
  }

  // Get measurement results

  if (meas_started)
  {
      if (digitalRead(PIN_TDC7200_INT) == LOW)
      {
        meas_started = false;

        for (uint8_t stop = 1; stop <= tdc7200_stops; ++stop)
        {
          uint64_t time;
          if (tof.readMeasurement(stop, time))
          {
            print_int64_t(time); 

            if (stop != tdc7200_stops)
              SerialUSB.print(",");
          }
        }
      
        SerialUSB.println();
      
      }
  }

  // Command interpreter

#define MAX_COMMAND_LEN 100
  static char cmd[MAX_COMMAND_LEN];
  static int cmd_len = 0;

  if (SerialUSB.available())
  {
    char c = SerialUSB.read();
    if ( (c == '\n') || (c == '\r') )
    {
      cmd[cmd_len] = 0;

      // Interpret

      if (cmd_len)
        cmd_interpret(cmd, cmd_len);

      cmd_len = 0;
    }
    else
    {
      if (cmd_len < MAX_COMMAND_LEN)
      {
        cmd[cmd_len] = c;
        cmd_len++;
      }
    }
  }


}
